package cz.mendelu.dnaAnalyser.analyse.rloopr;

import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.ArrayList;
import java.util.List;

public interface RloopMapper extends DataLocalisedMapper<RlooprResult> {

        // language=SQL
        String TABLE = "CREATE TABLE IF NOT EXISTS RLOOP (" +
                "ID integer NOT NULL AUTO_INCREMENT, " +
                "POSITION integer, " +
                "LENGTH integer, " +
                "RIZ varchar, " +
                "RIZGRICHNESS double, " + 
                "LINKER varchar, " +
                "REZ varchar, " +
                "MODEL varchar, " +
                "STRAND varchar, " +
                "RLOOPGRICHNESS double, " + 
                "G3 integer, " +
                "G4 integer, " +
                "GN integer, " +
                "PRIMARY KEY (ID))";

        // language=SQL
        @Select("SELECT count(*) FROM RLOOP")
        int countAll();

        // language=SQL
        @Select("SELECT * FROM RLOOP")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        ArrayList<RlooprResult> getAll();

        // language=SQL
        @Select("SELECT * FROM RLOOP WHERE POSITION > #{start} AND POSITION < #{end}")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        ArrayList<RlooprResult> getByWindow(Integer start, Integer end);

        // language=SQL
        @Select("SELECT * FROM RLOOP")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        Cursor<RlooprResult> getCursor();

        // language=SQL
        @Select("SELECT * FROM RLOOP WHERE ID = #{id}")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        RlooprResult getById(int id);

        // language=SQL
        @Update("UPDATE RLOOP SET " +
                "POSITION = #{position}, " +
                "LENGTH = #{length}, " +
                "RIZ = #{riz}, " +
                "RIZGRICHNESS = #{rizGRichness}, " +
                "LINKER = #{linker}, " +
                "REZ = #{rez} " +
                "MODEL = #{model} " +
                "STRAND = #{strand} " +
                "RLOOPGRICHNESS = #{rloopGRichness}, " +
                "G3 = #{g3cnt}, " +
                "G4 = #{g4cnt}, " +
                "GN = #{gNcnt}, " +
                "WHERE ID = #{id}")
        void update(RlooprResult result);

        // language=SQL
        @Delete("DELETE from RLOOP WHERE ID = #{id}")
        void delete(int id);

        // language=SQL
        @Insert("INSERT INTO RLOOP (ID, POSITION, LENGTH, RIZ, RIZGRICHNESS, LINKER, REZ, MODEL, STRAND, RLOOPGRICHNESS, G3, G4, GN)" +
                " VALUES " +
                "(#{id}, #{position}, #{length}, #{riz}, #{rizGRichness}, #{linker}, #{rez}, #{model}, #{strand}, #{rloopGRichness}, #{g3cnt}, #{g4cnt}, #{gNcnt})"
        )
        @Options(useGeneratedKeys = true, keyProperty = "id")
        void insert(RlooprResult result);

        // language=SQL
        @Select("SELECT count(*) FROM RLOOP WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition}")
        int countAllBetweenPosition(
                @Param("startPosition") int startPosition,
                @Param("endPosition") int endPosition);

        // language=SQL
        @Select("SELECT * FROM RLOOP WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        List<RlooprResult> getAllBetweenPositionOrderAsc(
                @Param("startPosition") int startPosition,
                @Param("endPosition") int endPosition,
                @Param("orderName") String orderName,
                RowBounds rowBounds);

        // language=SQL
        @Select("SELECT * FROM RLOOP WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} DESC")
        @Results(value = {
                @Result(property = "id", column = "ID"),
                @Result(property = "position", column = "POSITION"),
                @Result(property = "length", column = "LENGTH"),
                @Result(property = "riz", column = "RIZ"),
                @Result(property = "linker", column = "LINKER"),
                @Result(property = "rez", column = "REZ"),
                @Result(property = "strand", column = "STRAND"),
                @Result(property = "model", column = "MODEL"),
                @Result(property = "rizGRichness", column = "RIZGRICHNESS"),
                @Result(property = "rloopGRichness", column = "RLOOPGRICHNESS"),
                @Result(property = "g3cnt", column = "G3"),
                @Result(property = "g4cnt", column = "G4"),
                @Result(property = "gNcnt", column = "GN")
        })
        List<RlooprResult> getAllBetweenPositionOrderDesc(
                @Param("startPosition") int startPosition,
                @Param("endPosition") int endPosition,
                @Param("orderName") String orderName,
                RowBounds rowBounds);

}
