package cz.mendelu.dnaAnalyser.jwt;

import cz.mendelu.dnaAnalyser.utils.controller.GetMapping;
import cz.mendelu.dnaAnalyser.utils.controller.PostMapping;
import cz.mendelu.dnaAnalyser.utils.controller.PutMapping;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.AuthenticationException;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;

@RestController
@Api(tags = "api/jwt")
@RequestMapping("/api/jwt")
public class JwtController {

    @Value("${security.jwt.header}")
    private String tokenHeader;

    @Autowired
    private JwtService jwtService;

    @Data
    static public class LoginRequest {

        @NotNull
        @ApiModelProperty(
                example = "user@mendelu.cz",
                notes = "user login  as mail")
        private String login;

        @NotNull
        @ApiModelProperty(
                example = "user",
                notes = "user password")
        private String password;

    }

    @GetMapping(produces = MediaType.TEXT_PLAIN_VALUE)
    @ResponseStatus(HttpStatus.OK)
    @PreAuthorize("isAuthenticated()")
    @ApiOperation("Refresh and get authentication token")
    public String refreshAndGetAuthenticationToken(
            HttpServletRequest request
    ) {
        String oldToken = request.getHeader(tokenHeader);
        return jwtService.refreshToken(oldToken);
    }

    @PutMapping(produces = MediaType.TEXT_PLAIN_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation("Create Authentication token")
    public String createAuthenticationToken(
            @Valid @RequestBody LoginRequest request
    ) throws AuthenticationException
    {
        return jwtService.createToken(request.getLogin(), request.getPassword());
    }

    @PostMapping(
            consumes = {},
            produces = MediaType.TEXT_PLAIN_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation("Create host token token")
    public String createHostAuthenticationToken(
    ) {
        return jwtService.createHostToken();
    }


}
