package cz.mendelu.dnaAnalyser.utils.analyse.service

import cz.mendelu.dnaAnalyser.utils.analyse.data.TestDataMapper
import cz.mendelu.dnaAnalyser.utils.analyse.model.TestAnalyse
import spock.lang.Specification

class AbstractDataSessionServiceTest extends Specification {

    TestDataSessionService dataSessionService
    String tmpDir

    def setup() {
        tmpDir = File.createTempDir().getPath()
        dataSessionService = new TestDataSessionService(
                datasourceUrl: "jdbc:h2:file:$tmpDir/analyse-id;DB_CLOSE_ON_EXIT=FALSE",
                datasourceUsername: '',
                datasourcePassword: '',
                datasourceDriverClassName: 'org.h2.Driver'
        )
    }

    def "OpenDataSession"() {
        given:
        def analyze = new TestAnalyse(id: UUID.randomUUID())

        when:
        def session = dataSessionService.openDataSession(analyze)

        then:
        session != null
    }

    def "OnCreateTables"() {
        given:
        def analyze = new TestAnalyse(id: UUID.randomUUID())

        when:
        dataSessionService.openDataSession(analyze)

        then:
        dataSessionService.onCreateTables_called == true
        new File(tmpDir, analyze.id.toString() + ".mv.db").exists() == true
    }

    def "OnCreateTables when db exists"() {
        given:
        def analyze = new TestAnalyse(id: UUID.randomUUID())
        dataSessionService.openDataSession(analyze).close() // <-- DB file created
        dataSessionService.resetOnCallMarkers()

        when:
        dataSessionService.openDataSession(analyze)

        then:
        new File(tmpDir, analyze.id.toString() + ".mv.db").exists() == true
        dataSessionService.onCreateTables_called == false
    }

    def "OnConfiguration"() {
        given:
        def analyze = new TestAnalyse(id: UUID.randomUUID())

        when:
        def session = dataSessionService.openDataSession(analyze)

        then:
        dataSessionService.onConfiguration_called == true
        session.getMapper(TestDataMapper.class) != null
    }
}
