package cz.mendelu.dnaAnalyser.utils.analyse.service.heatmap

import cz.mendelu.dnaAnalyser.utils.analyse.data.TestData
import cz.mendelu.dnaAnalyser.utils.analyse.data.TestDataMapper
import cz.mendelu.dnaAnalyser.utils.analyse.model.Analyse
import cz.mendelu.dnaAnalyser.utils.analyse.model.TestAnalyse
import cz.mendelu.dnaAnalyser.sequence.Sequence
import spock.lang.Specification
import spock.lang.Unroll

class HeatmapServiceTest extends Specification {

    Analyse analyse
    HeatmapService heatmapService
    TestDataMapper dataMapper


    def setup() {
        analyse = new TestAnalyse(id: UUID.randomUUID(), finished: new Date())
        dataMapper = Mock(TestDataMapper)
        heatmapService = new TestHeatmapService(TestDataMapper.class, dataMapper)
    }

    @Unroll
    def "Simple heatmap test"() {
        setup:
        analyse.sequence = new Sequence(length: length)
        dataMapper.getAllBetweenPositionOrderAsc(from, to, "POSITION", _) >> testData

        when:
        def heatmap = heatmapService.heatmap(TestDataMapper.class, analyse, seg, from, to)

        then:
        heatmap.counts() == counts
        heatmap.coverages() == coverages
        heatmap.metaData.avg.count == avg(counts)
        heatmap.metaData.avg.coverage == avg(coverages)
        heatmap.metaData.min.count == min(counts)
        heatmap.metaData.min.coverage == min(coverages)
        heatmap.metaData.max.count == max(counts)
        heatmap.metaData.max.coverage == max(coverages)

        where:
        from | to | seg | length | counts          | coverages                 | testData
        0    | 10 | 5   | 10     | [0, 1, 0, 0, 0] | [0.0, 1.0, 1.0, 0.5, 0.0] | [new TestData(position: 2, length: 5)]
        0    | 10 | 5   | 10     | [0, 1, 1, 0, 0] | [0.0, 1.0, 1.0, 0.5, 0.0] | [new TestData(position: 2, length: 5), new TestData(position: 4, length: 3)]
    }

    def avg(list) { list.sum() / list.size() }
    def min(list) { list.min() }
    def max(list) { list.max() }

    @Unroll
    def "test calculateSegmentSize(sequenceSize: #sequenceSize, requestCount: #requestCount) = #segmentSize"() {
        expect:
        HeatmapService.calculateSegmentSize(sequenceSize, requestCount) == segmentSize

        where:
        sequenceSize | requestCount || segmentSize
        10           | 5            || 2
        10           | 6            || 2
        10           | 10           || 1
        10           | 15           || 1
    }

    @Unroll
    def "test calculateSegmentsCount(sequenceSize: #sequenceSize, requestCount: #requestCount, segmentSize: #segmentSize) = #segmentsCount"() {
        expect:
        HeatmapService.calculateSegmentsCount(sequenceSize, requestCount, segmentSize) == segmentsCount

        where:
        sequenceSize | requestCount | segmentSize || segmentsCount
        10           | 5            | 2           || 5
        10           | 6            | 2           || 5
        10           | 10           | 1           || 10
        10           | 15           | 1           || 10
    }

    @Unroll
    def "test segmentIndex(position: #position, segmentSize: #segmentSize, segmentsCount: #segmentsCount) = #index"() {
        expect:
        HeatmapService.segmentIndex(position, segmentSize, segmentsCount) == index

        where:
        position | segmentSize | segmentsCount | index
        1        | 5           | 2             | 0
        6        | 5           | 2             | 1
        12       | 5           | 2             | 0
    }

}
