package cz.mendelu.dnaAnalyser.utils.service

import cz.mendelu.dnaAnalyser.utils.exception.InternalServerException
import cz.mendelu.dnaAnalyser.sequence.Sequence
import cz.mendelu.dnaAnalyser.sequence.SequenceType
import spock.lang.Shared
import spock.lang.Specification

class JsonServiceTest extends Specification {

    @Shared JsonService jsonService = new JsonService()

    def "test parse Sequence from json"() {
        given:
        def json = """{
            "id": "090e5461-6dd3-44ff-8e01-7b84435857fa",
            "name": "Shigella flexneri 2a str. 301 chromosome",
            "created": 1519116472801,
            "type": "DNA",
            "circular": false,
            "length": 826,
            "tags": [
                "Shigella flexneri"
            ]
        }"""

        when:
        def result = jsonService.fromJson(json, Sequence)

        then:
        result != null
        result.id == UUID.fromString('090e5461-6dd3-44ff-8e01-7b84435857fa')
        result.name == 'Shigella flexneri 2a str. 301 chromosome'
        result.created == new Date(1519116472801)
        result.type == SequenceType.DNA
        result.circular == false
        result.length == 826
        result.tags == ['Shigella flexneri'] as Set
    }

    def "test parse nonsense"() {
        given:
        def json = """{
            "abc": "xyz"
        }"""

        when:
        jsonService.fromJson(json, Sequence)

        then:
        thrown(InternalServerException)
    }
}
